"""Translator for messages (from admonition)."""

from typing import Optional, TypedDict

from docutils import nodes
from sphinx.locale import admonitionlabels
from sphinx.writers.html5 import HTML5Translator


class MessageClassMap(TypedDict):
    """Mappings definition for admonitions and color classes.

    :ref: https://www.docutils.org/docs/ref/rst/directives.html#specific-admonitions
    """

    # docutils admonition's types
    attention: str
    caution: str
    danger: str
    error: str
    hint: str
    important: str
    note: str
    tip: str
    warning: str
    # Sphinx extra admonitions
    Todo: str


DEFAULT_MESSAGE_CLASSES: MessageClassMap = {
    # docutils admonition's types
    "attention": "is-warning",
    "caution": "is-warning",
    "danger": "is-danger",
    "error": "is-danger",
    "hint": "is-info",
    "important": "is-info",
    "note": "is-info",
    "tip": "is-info",
    "warning": "is-warning",
    # Sphinx extra admonitions
    "Todo": "is-link",
}


def visit_admonition(  # noqa: D103
    self: HTML5Translator, node: nodes.admonition, name: str = ""
) -> None:
    message_classes: MessageClassMap = self.builder.app.config.bulma_message_classes
    fallback_class = self.builder.app.config.bulma_message_fallback
    if isinstance(node.children[0], nodes.title):
        msg_title = node.pop(0).astext()
        msg_class = message_classes.get(msg_title, fallback_class)
    else:
        msg_title = admonitionlabels[name]
        msg_class = message_classes.get(name, fallback_class)
    self.body.append(f'<article class="message {msg_class}">')
    self.body.append(f"""
      <div class="message-header">
        <p>{msg_title}</p>
      </div>
    """)
    self.body.append('  <div class="message-body">')


def depart_admonition(  # noqa: D103
    self: HTML5Translator, node: Optional[nodes.admonition] = None
) -> None:
    self.body.append("  </div>")
    self.body.append("</article>")



"""Bluma using suite for Sphinx.."""

from sphinx.application import Sphinx

from .components.messages import DEFAULT_MESSAGE_CLASSES, MessageClassMap
from .translator import BulmaTranslator

__version__ = "0.2.1"


def setup(app: Sphinx):  # noqa: D103
    app.add_config_value(
        "bulma_message_classes",
        DEFAULT_MESSAGE_CLASSES,
        "env",
        MessageClassMap,
    )
    app.add_config_value("bulma_message_fallback", "", "env", str)
    app.set_translator("html", BulmaTranslator)
    app.set_translator("dirhtml", BulmaTranslator)
    return {
        "version": __version__,
        "env_version": 1,
        "parallel_read_safe": True,
        "parallel_write_safe": True,
    }



"""Additional for navbar."""

from typing import Any, Optional

from docutils import nodes
from sphinx.addnodes import toctree
from sphinx.application import Sphinx


def register_root_toctree_dict(
    app: Sphinx,
    pagename: str,
    templatename: str,
    context: dict[str, Any],
    doctree: Optional[nodes.document],
) -> None:
    """Pass toctree of root for navbar."""

    def root_toctree_dict(show_hidden: bool = False):
        doctree = app.env.get_doctree(app.config.root_doc)
        subs = []
        for tree in doctree.findall(toctree):
            if tree["hidden"] and not show_hidden:
                continue
            for title, page in tree["entries"]:
                subs.append(
                    {
                        "uri": app.builder.get_relative_uri(pagename, page),
                        "title": title
                        or next(
                            app.env.get_doctree(page).findall(nodes.title)
                        ).astext(),
                    }
                )
        return subs

    context["root_toctree_dict"] = root_toctree_dict
    return



"""Standard tests."""

from io import StringIO

import pytest
from sphinx.testing.util import SphinxTestApp


@pytest.mark.sphinx("html")
def test__it(app: SphinxTestApp, status: StringIO, warning: StringIO):
    """Test to pass."""
    app.build()
